# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""Customize generated code here.

Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
import base64
import json
from typing import List, Any, Tuple, Union, Optional, Callable, cast


from threading import Lock
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePrivateKey
from cryptography.x509 import load_pem_x509_certificate, Certificate
from typing_extensions import Self

from azure.core.exceptions import raise_with_traceback  # pylint: disable=no-raise-with-traceback
from azure.core.tracing.decorator import distributed_trace
from azure.core.configuration import Configuration
from azure.core.credentials import TokenCredential
from azure.security.attestation.models import OpenIDConfigurationResponse


from ._client import AttestationClient as AzureAttestationRestClient
from .models import (  # pylint: disable=reimported
    AttestationResult as GeneratedAttestationResult,
    RuntimeData,
    InitTimeData,
    DataType,
    AttestSgxEnclaveRequest,
    AttestOpenEnclaveRequest,
    AttestationResult,
    TpmAttestationResult,
    PolicyResult as GeneratedPolicyResult,
    PolicyCertificatesResult as GeneratedPolicyCertificatesResult,
    JsonWebKey as JSONWebKey,
    AttestationCertificateManagementBody,
    StoredAttestationPolicy as GeneratedStoredAttestationPolicy,
    PolicyCertificatesModificationResult as GeneratedPolicyCertificatesModificationResult,
    AttestationPolicyResult,
    AttestationPolicyCertificateResult,
)

from .models._patch import (
    AttestationTokenValidationException,
    AttestationPolicyToken,
    AttestationSigner,
    AttestationToken,
)

from .models._enums import (
    AttestationType,
)

from ._common import pem_from_base64, validate_signing_keys, merge_validation_args, base64url_decode


class AttestationClient:
    # pylint: disable=protected-access
    """Provides access to the Attestation family of APIs for trusted environment attestation.

    :param str endpoint: The attestation instance base URI, for example https://mytenant.attest.azure.net.
    :param credential: Credentials for the caller used to interact with the service.
    :type credential: :class:`~azure.core.credentials.TokenCredential`
    :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
    :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
        if the token is invalid, the `validation_callback` function should throw
        an exception.
    :paramtype validation_callback: ~typing.Callable[[AttestationToken, AttestationSigner], None]
    :keyword bool validate_signature: If True, validate the signature of the token being validated.
    :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
    :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
    :keyword float validation_slack: Slack time for validation - tolerance applied
        to help account for clock drift between the issuer and the current machine.
    :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
    :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.
    :keyword str api_version: The API version to use for the request.

    .. tip::
        The `validate_token`, `validation_callback`, `validate_signature`,
        `validate_expiration`, `validate_not_before_time`, `validate_issuer`, and
        `issuer` keyword arguments are default values applied to each API call within
        the :py:class:`AttestationClient` class. These values can be
        overridden on individual API calls as needed.

    For additional client creation configuration options, please see `Python Request
    Options <https://aka.ms/azsdk/python/options>`_.

    """

    def __init__(self, endpoint: str, credential: TokenCredential, **kwargs) -> None:
        if not credential:
            raise ValueError("Missing credential.")
        self._config = AttestationClientConfiguration(**kwargs)
        self._client = AzureAttestationRestClient(endpoint, credential, **kwargs)
        self._statelock = Lock()
        self._signing_certificates: list[AttestationSigner] = []

    @distributed_trace
    def get_open_id_metadata(self, **kwargs: Any) -> OpenIDConfigurationResponse:
        """Retrieves the OpenID metadata configuration document for this attestation instance.

        The metadata configuration document is defined in the `OpenID Connect
        Discovery <https://openid.net/specs/openid-connect-discovery-1_0.html#ProviderConfigurationResponse>`_
        specification.

        The attestation service currently returns the following fields:

        * issuer
        * jwks_uri
        * claims_supported

        :return: OpenID metadata configuration
        :rtype: Dict[str, Any]
        """
        return self._client.metadata_configuration.get(**kwargs)

    @distributed_trace
    def get_signing_certificates(self, **kwargs) -> List[AttestationSigner]:
        """Returns the set of signing certificates used to sign attestation tokens.

        :return: A list of :class:`azure.security.attestation.AttestationSigner` objects.

        :rtype: List[~azure.security.attestation.AttestationSigner]

        For additional request configuration options, please see `Python Request
        Options <https://aka.ms/azsdk/python/options>`_.

        """
        signing_certificates = self._client.signing_certificates.get(**kwargs)
        return [AttestationSigner._from_generated(key) for key in signing_certificates["keys"]]  # type: ignore

    @distributed_trace
    def attest_sgx_enclave(
        self,
        quote: bytes,
        *,
        inittime_data: Optional[bytes] = None,
        inittime_json: Optional[bytes] = None,
        runtime_data: Optional[bytes] = None,
        runtime_json: Optional[bytes] = None,
        draft_policy: Optional[str] = None,
        validate_token: Optional[bool] = None,
        validation_callback: Optional[Callable[["AttestationToken", "AttestationSigner"], None]] = None,
        validate_signature: Optional[bool] = None,
        validate_expiration: Optional[bool] = None,
        issuer: Optional[str] = None,
        validation_slack: Optional[float] = None,
        validate_issuer: Optional[bool] = None,
        validate_not_before_time: Optional[bool] = None,
        **kwargs
    ) -> Tuple[AttestationResult, AttestationToken]:
        """Attests the validity of an SGX quote.

        :param bytes quote: An SGX quote generated from an Intel(tm) SGX enclave
        :keyword bytes inittime_data: Data presented at the time that the SGX
            enclave was initialized.
        :keyword bytes inittime_json: Data presented at the time that the SGX
            enclave was initialized, JSON encoded.
        :keyword bytes runtime_data: Data presented at the time that the open_enclave
            report was created.
        :keyword bytes runtime_json: Data presented at the time that the open_enclave
            report was created. JSON Encoded.
        :keyword str draft_policy: "draft" or "experimental" policy to be used with
            this attestation request. If this parameter is provided, then this
            policy document will be used for the attestation request.
            This allows a caller to test various policy documents against actual data
            before applying the policy document via the set_policy API
        :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
            if the token is invalid, the `validation_callback` function should throw
            an exception.
        :paramtype validation_callback: ~typing.Callable[[AttestationToken, AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword float validation_slack: Slack time for validation - tolerance applied
            to help account for clock drift between the issuer and the current machine.
        :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.


        :return: :class:`AttestationResult` containing the claims in the returned attestation token.

        :rtype: Tuple[~azure.security.attestation.AttestationResult, ~azure.security.attestation.AttestationToken]

        .. note::
            Note that if the `draft_policy` parameter is provided, the resulting
            attestation token will be an unsecured attestation token.

        .. admonition:: Example:

            .. literalinclude:: ../samples/sample_attest_enclave.py
                :start-after: [START attest_sgx_enclave_shared]
                :end-before: [END attest_sgx_enclave_shared]
                :language: python
                :dedent: 8
                :caption: Attesting an SGX Enclave

        For additional request configuration options, please see `Python Request
        Options <https://aka.ms/azsdk/python/options>`_.

        """

        # Handle explicit validation parameters
        validation_args = {
            "validate_token": validate_token,
            "validation_callback": validation_callback,
            "validate_signature": validate_signature,
            "validate_expiration": validate_expiration,
            "issuer": issuer,
            "validation_slack": validation_slack,
            "validate_issuer": validate_issuer,
            "validate_not_before_time": validate_not_before_time,
        }
        # Remove None values
        validation_args = {k: v for k, v in validation_args.items() if v is not None}
        # Add validation args to kwargs for merge_validation_args
        kwargs.update(validation_args)

        if inittime_json and inittime_data:
            raise ValueError("Cannot provide both inittime_json and inittime_data.")
        if runtime_json and runtime_data:
            raise ValueError("Cannot provide both runtime_data and runtime_json.")

        # If the input was JSON, make sure that it's valid JSON before sending it
        # to the service.
        if inittime_json:
            try:
                json.loads(inittime_json)
            except json.JSONDecodeError:
                raise_with_traceback(ValueError, "Content must be valid JSON.")

        if runtime_json:
            try:
                json.loads(runtime_json)
            except json.JSONDecodeError:
                raise_with_traceback(ValueError, "Content must be valid JSON.")

        runtime = None
        if runtime_data:
            runtime = RuntimeData(data=runtime_data, data_type=DataType.BINARY)

        if runtime_json:
            runtime = RuntimeData(data=runtime_json, data_type=DataType.JSON)

        inittime = None
        if inittime_data:
            inittime = InitTimeData(data=inittime_data, data_type=DataType.BINARY)
        if inittime_json:
            inittime = InitTimeData(data=inittime_data, data_type=DataType.JSON)

        request = AttestSgxEnclaveRequest(
            quote=quote,
            init_time_data=inittime,
            runtime_data=runtime,
            draft_policy_for_attestation=draft_policy,
        )

        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        options = merge_validation_args(self._config._kwargs, kwargs)

        result = self._client.attestation.attest_sgx_enclave(request, **kwargs)
        token = AttestationToken(token=result.token, body_type=GeneratedAttestationResult)

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)

        return (
            AttestationResult._from_generated(token._get_body()),  # type: ignore
            token,
        )

    @distributed_trace
    def attest_open_enclave(
        self,
        report: bytes,
        *,
        inittime_data: Optional[bytes] = None,
        inittime_json: Optional[bytes] = None,
        runtime_data: Optional[bytes] = None,
        runtime_json: Optional[bytes] = None,
        draft_policy: Optional[str] = None,
        validate_token: Optional[bool] = None,
        validation_callback: Optional[Callable[["AttestationToken", "AttestationSigner"], None]] = None,
        validate_signature: Optional[bool] = None,
        validate_expiration: Optional[bool] = None,
        issuer: Optional[str] = None,
        validation_slack: Optional[float] = None,
        validate_issuer: Optional[bool] = None,
        validate_not_before_time: Optional[bool] = None,
        **kwargs
    ) -> Tuple[AttestationResult, AttestationToken]:
        """Attests the validity of an Open Enclave report.

        :param bytes report: An open_enclave report generated from an Intel(tm)
            SGX enclave
        :keyword bytes inittime_data: Data presented at the time that the SGX
            enclave was initialized.
        :keyword bytes inittime_json: Data presented at the time that the SGX
            enclave was initialized, JSON encoded.
        :keyword bytes runtime_data: Data presented at the time that the open_enclave
            report was created.
        :keyword bytes runtime_json: Data presented at the time that the open_enclave
            report was created. JSON Encoded.
        :keyword str draft_policy: "draft" or "experimental" policy to be used with
            this attestation request. If this parameter is provided, then this
            policy document will be used for the attestation request.
            This allows a caller to test various policy documents against actual data
            before applying the policy document via the set_policy API.
        :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
            if the token is invalid, the `validation_callback` function should throw
            an exception.
        :paramtype validation_callback: ~typing.Callable[[AttestationToken, AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword float validation_slack: Slack time for validation - tolerance applied
            to help account for clock drift between the issuer and the current machine.
        :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.

        :return: :class:`AttestationResult` containing the claims in the returned attestation token.

        :rtype: Tuple[~azure.security.attestation.AttestationResult, ~azure.security.attestation.AttestationToken]

        .. admonition:: Example: Simple OpenEnclave attestation.

            .. literalinclude:: ../samples/sample_attest_enclave.py
                :start-after: [START attest_open_enclave_shared]
                :end-before: [END attest_open_enclave_shared]
                :language: python
                :dedent: 8
                :caption: Attesting an open_enclave report for an SGX enclave.

        .. admonition:: Example: Simple OpenEnclave attestation with draft attestation policy.

            .. literalinclude:: ../samples/sample_attest_enclave.py
                :start-after: [START attest_open_enclave_shared_draft]
                :end-before: [END attest_open_enclave_shared_draft]
                :language: python
                :dedent: 8
                :caption: Attesting using a draft attestation policy.

        .. note::
            Note that if the `draft_policy` parameter is provided, the resulting
            attestation token will be an unsecured attestation token.

        For additional request configuration options, please see `Python Request
        Options <https://aka.ms/azsdk/python/options>`_.

        """

        # Handle explicit validation parameters
        validation_args = {
            "validate_token": validate_token,
            "validation_callback": validation_callback,
            "validate_signature": validate_signature,
            "validate_expiration": validate_expiration,
            "issuer": issuer,
            "validation_slack": validation_slack,
            "validate_issuer": validate_issuer,
            "validate_not_before_time": validate_not_before_time,
        }
        # Remove None values
        validation_args = {k: v for k, v in validation_args.items() if v is not None}
        # Add validation args to kwargs for merge_validation_args
        kwargs.update(validation_args)

        if inittime_json and inittime_data:
            raise ValueError("Cannot provide both inittime_json and inittime_data.")
        if runtime_json and runtime_data:
            raise ValueError("Cannot provide both runtime_data and runtime_json.")

        # If the input was JSON, make sure that it's valid JSON before sending it
        # to the service.
        if inittime_json:
            try:
                json.loads(inittime_json)
            except json.JSONDecodeError:
                raise_with_traceback(ValueError, "Content must be valid JSON.")

        if runtime_json:
            try:
                json.loads(runtime_json)
            except json.JSONDecodeError:
                raise_with_traceback(ValueError, "Content must be valid JSON.")

        # Now create the RuntimeData object to be sent to the service.
        runtime = None
        if runtime_data:
            runtime = RuntimeData(data=runtime_data, data_type=DataType.BINARY)

        if runtime_json:
            runtime = RuntimeData(data=runtime_json, data_type=DataType.JSON)

        # And the InitTimeData object to be sent to the service.
        inittime = None
        if inittime_data:
            inittime = InitTimeData(data=inittime_data, data_type=DataType.BINARY)
        if inittime_json:
            inittime = InitTimeData(data=inittime_data, data_type=DataType.JSON)

        request = AttestOpenEnclaveRequest(
            report=report,
            init_time_data=inittime,
            runtime_data=runtime,
            draft_policy_for_attestation=draft_policy,
        )

        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        options = merge_validation_args(self._config._kwargs, kwargs)

        result = self._client.attestation.attest_open_enclave(request, **kwargs)
        token = AttestationToken(token=result.token, body_type=GeneratedAttestationResult)

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)
        return (AttestationResult._from_generated(token._get_body()), token)  # type: ignore

    @distributed_trace
    def attest_tpm(self, content: bytes, **kwargs) -> TpmAttestationResult:
        """Performs TPM attestation.

        :param bytes content: Attestation request for Trusted Platform Module (TPM) attestation.
        :returns: Attestation response for Trusted Platform Module (TPM) attestation.
        :rtype: ~azure.security.attestation.TpmAttestationResult
        """

        response = self._client.attestation.attest_tpm(data=content, **kwargs)
        # Handle response.data as either bytes or base64-encoded string
        data = response.data
        if isinstance(data, str):
            # If it's a base64-encoded string, decode it to bytes
            # Use base64url_decode for proper handling of URL-safe base64
            data = base64url_decode(data)
        result = TpmAttestationResult(cast(bytes, data))
        return result

    def _get_signers(self, **kwargs: Any) -> list[AttestationSigner]:
        """Returns the set of signing certificates used to sign attestation tokens.

        :return: A list of attestation signers used to validate tokens.
        :rtype: list[AttestationSigner]
        """

        with self._statelock:
            if not self._signing_certificates:
                signing_certificates = self._client.signing_certificates.get(**kwargs)
                self._signing_certificates = []
                for key in signing_certificates["keys"]:
                    # Convert the returned certificate chain into an array of X.509 Certificates.
                    self._signing_certificates.append(AttestationSigner._from_generated(key))  # type: ignore
            signers = self._signing_certificates
        return signers

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details) -> None:
        self._client.__exit__(*exc_details)


class AttestationAdministrationClient:
    # pylint: disable = line-too-long, protected-access
    """Provides administrative APIs for managing an instance of the Attestation Service.

    The :class:`~azure.security.attestation.AttestationAdministrationClient` object implements the policy
    management and policy certificate management functions.

    :param str endpoint: The attestation instance base URI, for example https://mytenant.attest.azure.net.
    :param credential: Credentials for the caller used to interact with the service.
    :type credential: :class:`~azure.core.credentials.TokenCredential`
    :keyword str signing_key: PEM encoded signing key to be used for all
        operations.
    :keyword str signing_certificate: PEM encoded X.509 certificate to be used for all
        operations.
    :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
    :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
        if the token is invalid, the `validation_callback` function should throw
        an exception.
    :paramtype validation_callback: ~typing.Callable[[~azure.security.attestation.AttestationToken, ~azure.security.attestation.AttestationSigner], None]
    :keyword bool validate_signature: If True, validate the signature of the token being validated.
    :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
    :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
    :keyword float validation_slack: Slack time for validation - tolerance applied
        to help account for clock drift between the issuer and the current machine.
    :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
    :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.
    :keyword str api_version: The API version to use for the request.

    If the `signing_key` and `signing_certificate` parameters
    are provided, they will be applied to the following APIs:

    * :py:func:`set_policy`
    * :py:func:`reset_policy`
    * :py:func:`add_policy_management_certificate`
    * :py:func:`remove_policy_management_certificate`

    .. note::
        The `signing_key` and `signing_certificate` parameters are a pair. If one
        is present, the other must also be provided. In addition, the public key
        in the `signing_key` and the public key in the `signing_certificate` must
        match to ensure that the `signing_certificate` can be used to validate an
        object signed by `signing_key`.

    .. tip::
        The `validate_token`, `validation_callback`, `validate_signature`,
        `validate_expiration`, `validate_not_before_time`, `validate_issuer`, and
        `issuer` keyword arguments are default values applied to each API call within
        the :py:class:`AttestationAdministrationClient` class. These values can be
        overridden on individual API calls as needed.

    For additional client creation configuration options, please see `Python Request
    Options <https://aka.ms/azsdk/python/options>`_.

    """

    def __init__(self, endpoint: str, credential: TokenCredential, **kwargs: Any) -> None:
        if not credential:
            raise ValueError("Missing credential.")
        self._config = AttestationClientConfiguration(**kwargs)
        self._client = AzureAttestationRestClient(endpoint, credential, **kwargs)
        self._statelock = Lock()
        self._signing_certificates: list[AttestationSigner] = []

        self._signing_key = None
        self._signing_certificate = None

        signing_key = kwargs.pop("signing_key", None)
        signing_certificate = kwargs.pop("signing_certificate", None)
        if signing_key or signing_certificate:
            self._signing_key, self._signing_certificate = validate_signing_keys(signing_key, signing_certificate)

    @distributed_trace
    def get_policy(
        self,
        attestation_type: Union[str, AttestationType],
        *,
        validation_slack: float = 0.5,
        validate_signature: bool = True,
        validate_issuer: bool = True,
        validation_callback: Optional[Callable[[AttestationToken, AttestationSigner], None]] = None,
        validate_expiration: bool = True,
        issuer: Optional[str] = None,
        validate_token: bool = True,
        validate_not_before_time: bool = True,
        **kwargs: Any
    ) -> Tuple[str, AttestationToken]:
        """Retrieves the attestation policy for a specified attestation type.

        :param attestation_type: :class:`azure.security.attestation.AttestationType` for
            which to retrieve the policy.
        :type attestation_type: Union[str, ~azure.security.attestation.AttestationType]
        :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
            if the token is invalid, the `validation_callback` function should throw
            an exception.
        :paramtype validation_callback: ~typing.Callable[[~azure.security.attestation.AttestationToken, ~azure.security.attestation.AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword float validation_slack: Slack time for validation - tolerance applied
            to help account for clock drift between the issuer and the current machine.
        :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.

        :return: A tuple containing the attestation policy and the token returned
            by the service..

        :rtype: Tuple[str, ~azure.security.attestation.AttestationToken]

        :raises ~azure.security.attestation.AttestationTokenValidationException: Raised
            when an attestation token is invalid.

        .. note::
            The Azure Attestation Policy language is defined `here
            <https://docs.microsoft.com/azure/attestation/author-sign-policy>`_

        .. admonition:: Example: Retrieving the current policy on an attestation instance.

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [BEGIN get_policy]
                :end-before: [END get_policy]
                :language: python
                :dedent: 8
                :caption: Getting the current policy document.

        """
        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        validation_args = {
            "validation_slack": validation_slack,
            "validate_signature": validate_signature,
            "validate_issuer": validate_issuer,
            "validation_callback": validation_callback,
            "validate_expiration": validate_expiration,
            "issuer": issuer,
            "validate_token": validate_token,
            "validate_not_before_time": validate_not_before_time,
        }
        # Merge validation_args with kwargs to include any additional parameters
        validation_args.update(kwargs)
        options = merge_validation_args(self._config._kwargs, validation_args)

        policyResult = self._client.policy.get(attestation_type, **kwargs)
        token = AttestationToken(token=policyResult.token, body_type=GeneratedPolicyResult)
        token_body = token._get_body()
        stored_policy = AttestationToken(token=token_body.policy, body_type=GeneratedStoredAttestationPolicy)

        policy_body = stored_policy._get_body()
        actual_policy = policy_body.attestation_policy if policy_body else "".encode("ascii")  # type: bytes

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)

        if isinstance(actual_policy, bytes):
            return actual_policy.decode("utf-8"), token

        return actual_policy, token

    @distributed_trace
    def set_policy(
        self,
        attestation_type: Union[str, AttestationType],
        attestation_policy: str,
        *,
        validation_slack: float = 0.5,
        validate_not_before_time: bool = True,
        validation_callback: Optional[Callable[[AttestationToken, AttestationSigner], None]] = None,
        issuer: Optional[str] = None,
        signing_key: Optional[Union[str, RSAPrivateKey, EllipticCurvePrivateKey]] = None,
        signing_certificate: Optional[Union[str, Certificate]] = None,
        validate_issuer: bool = True,
        validate_signature: bool = True,
        validate_token: bool = True,
        validate_expiration: bool = True,
        **kwargs: Any
    ) -> Tuple[AttestationPolicyResult, AttestationToken]:
        """Sets the attestation policy for the specified attestation type.

        :param attestation_type: :class:`azure.security.attestation.AttestationType` for
            which to set the policy.
        :type attestation_type: Union[str, ~azure.security.attestation.AttestationType]
        :param str attestation_policy: Attestation policy to be set.
        :keyword str signing_key: PEM encoded signing key to be used to sign the policy
            before sending it to the service.
        :keyword str signing_certificate: PEM encoded X.509 certificate to be sent to the
            service along with the policy.
        :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
            if the token is invalid, the `validation_callback` function should throw
            an exception.
        :paramtype validation_callback: ~typing.Callable[[~azure.security.attestation.AttestationToken, ~azure.security.attestation.AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword float validation_slack: Slack time for validation - tolerance applied
            to help account for clock drift between the issuer and the current machine.
        :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.

        :return: Result of set policy operation.

        :rtype: Tuple[~azure.security.attestation.AttestationPolicyResult, ~azure.security.attestation.AttestationToken]

        :raises ~azure.security.attestation.AttestationTokenValidationException: Raised
            when an attestation token is invalid.

        .. admonition:: Example: Setting the attestation policy on an AAD mode
            attestation instance (no signing key required).

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [BEGIN set_policy_unsecured]
                :end-before: [END set_policy_unsecured]
                :language: python
                :dedent: 0
                :caption: Setting a security policy without a signing key.

        .. admonition:: Example: Setting the attestation policy and verifying
            that the policy was received by the service.

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [START validate_policy_hash]
                :end-before: [END validate_policy_hash]
                :language: python
                :dedent: 0
                :caption: Setting the attestation policy with hash verification.

        .. note::
            If the attestation instance is in *Isolated* mode, then the
            `signing_key` parameter MUST be a signing key containing one of the
            certificates returned by :meth:`get_policy_management_certificates`.

            If the attestation instance is in *AAD* mode, then the `signing_key`
            parameter does not need to be provided.

        """

        # If the caller provided a signing key and certificate, validate that,
        # otherwise use the default values from the service.

        # Note that the signing_key and signing_certificate argument to AttestationToken
        # is polymorphic - it can be either a PEM encoded string or a cryptography
        # object. The self._signing_key and self._signing_certificate are cryptography
        # objects, the keyword arguments are strings. The AttestationToken knows
        # how to tease these two apart.
        if not signing_key:
            signing_key = self._signing_key
        if not signing_certificate:
            signing_certificate = self._signing_certificate

        policy_token = AttestationToken(
            body=GeneratedStoredAttestationPolicy(attestation_policy=attestation_policy.encode("ascii")),
            signing_key=signing_key,
            signing_certificate=signing_certificate,
            body_type=GeneratedStoredAttestationPolicy,
        )

        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        validation_args = {
            "validation_slack": validation_slack,
            "validate_not_before_time": validate_not_before_time,
            "validation_callback": validation_callback,
            "issuer": issuer,
            "validate_issuer": validate_issuer,
            "validate_signature": validate_signature,
            "validate_token": validate_token,
            "validate_expiration": validate_expiration,
        }
        # Merge validation_args with kwargs to include any additional parameters
        validation_args.update(kwargs)
        options = merge_validation_args(self._config._kwargs, validation_args)

        policyResult = self._client.policy.set(
            attestation_type=attestation_type, new_attestation_policy=policy_token.to_jwt_string(), **kwargs
        )
        token = AttestationToken(token=policyResult.token, body_type=GeneratedPolicyResult)

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)

        return (
            AttestationPolicyResult._from_generated(token._get_body()),  # type: ignore
            token,
        )

    @distributed_trace
    def reset_policy(
        self,
        attestation_type: Union[str, AttestationType],
        *,
        validation_callback: Optional[Callable[[AttestationToken, AttestationSigner], None]] = None,
        validate_issuer: bool = True,
        validate_token: bool = True,
        validation_slack: float = 0.5,
        signing_key: Optional[str] = None,
        signing_certificate: Optional[str] = None,
        issuer: Optional[str] = None,
        validate_signature: bool = True,
        validate_not_before_time: bool = True,
        validate_expiration: bool = True,
        **kwargs: Any
    ) -> Tuple[AttestationPolicyResult, AttestationToken]:
        """Resets the attestation policy for the specified attestation type to the default value.

        :param attestation_type: :class:`azure.security.attestation.AttestationType` for
            which to set the policy.
        :type attestation_type: Union[str, ~azure.security.attestation.AttestationType]
        :keyword str signing_key: PEM encoded signing key to be used to sign the policy
            before sending it to the service.
        :keyword str signing_certificate: PEM encoded X.509 certificate to be sent to the
            service along with the policy.
        :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
            if the token is invalid, the `validation_callback` function should throw
            an exception.
        :paramtype validation_callback: ~typing.Callable[[~azure.security.attestation.AttestationToken, ~azure.security.attestation.AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword float validation_slack: Slack time for validation - tolerance applied
            to help account for clock drift between the issuer and the current machine.
        :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.

        :return: A policy set result reflecting the outcome of the policy removal and
            the token which contained the result.

        :rtype: Tuple[~azure.security.attestation.AttestationPolicyResult, ~azure.security.attestation.AttestationToken]

        :raises ~azure.security.attestation.AttestationTokenValidationException: Raised
            when an attestation token is invalid.

        .. note::
            If the attestation instance is in *Isolated* mode, then the
            `signing_key` parameter MUST be a signing key containing one of the
            certificates returned by :meth:`get_policy_management_certificates`.

            If the attestation instance is in *AAD* mode, then the `signing_key`
            parameter does not need to be provided.

        .. admonition:: Example: Resetting the attestation policy on an AAD mode
            attestation instance (no signing key required).

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [BEGIN reset_aad_policy]
                :end-before: [END reset_aad_policy]
                :language: python
                :dedent: 8
                :caption: Resetting an AAD mode attestation instance.

        .. admonition:: Example: Resetting the attestation policy on an Isolated mode
            attestation instance (signing key required).

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [BEGIN reset_isolated_policy]
                :end-before: [END reset_isolated_policy]
                :language: python
                :dedent: 8
                :caption: Resetting an AAD mode attestation instance.

        """

        # If the caller provided a signing key and certificate, validate that,
        # otherwise use the default values from the service.
        if not signing_key:
            signing_key = self._signing_key  # type: ignore
        if not signing_certificate:
            signing_certificate = self._signing_certificate  # type: ignore

        policy_token = AttestationToken(body=None, signing_key=signing_key, signing_certificate=signing_certificate)

        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        validation_args = {
            "validation_callback": validation_callback,
            "validate_issuer": validate_issuer,
            "validate_token": validate_token,
            "validation_slack": validation_slack,
            "issuer": issuer,
            "validate_signature": validate_signature,
            "validate_not_before_time": validate_not_before_time,
            "validate_expiration": validate_expiration,
        }
        # Merge validation_args with kwargs to include any additional parameters
        validation_args.update(kwargs)
        options = merge_validation_args(self._config._kwargs, validation_args)

        policyResult = self._client.policy.reset(
            attestation_type=attestation_type, policy_jws=policy_token.to_jwt_string(), **kwargs
        )
        token = AttestationToken(token=policyResult.token, body_type=GeneratedPolicyResult)

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)

        return (AttestationPolicyResult._from_generated(token._get_body()), token)  # type: ignore

    @distributed_trace
    def get_policy_management_certificates(
        self,
        *,
        validate_issuer: bool = True,
        validate_token: bool = True,
        validate_expiration: bool = True,
        validate_not_before_time: bool = True,
        issuer: Optional[str] = None,
        validation_slack: float = 0.5,
        validate_signature: bool = True,
        validation_callback: Optional[Callable[[AttestationToken, AttestationSigner], None]] = None,
        **kwargs: Any
    ) -> Tuple[List[List[str]], AttestationToken]:
        """Retrieves the set of policy management certificates for the instance.

        The list of policy management certificates will only have values if the
        attestation service instance is in Isolated mode.

        :keyword bool validate_token: If True, validate the token, otherwise
            return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to
            perform custom validation of the token. If the token is invalid,
            the `validation_callback` function should throw an exception to cause
            the API call to fail.
        :paramtype validation_callback: ~typing.Callable[[~azure.security.attestation.AttestationToken, ~azure.security.attestation.AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the
            token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time
            of the token being validated.
        :keyword float validation_slack: Slack time for validation - tolerance
            applied to help account for clock drift between the issuer and
            the current machine.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword bool validate_issuer: If True, validate that the issuer of the
            token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the
            "Not Before" time in the token.

        :return: A tuple containing the list of PEM encoded X.509 certificate chains and an attestation token.
        :rtype: Tuple[List[List[str]], ~azure.security.attestation.AttestationToken]

        .. admonition:: Example: Retrieving the set of policy management certificates
            for an isolated attestation instance.

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [BEGIN get_policy_management_certificate]
                :end-before: [END get_policy_management_certificate]
                :language: python
                :dedent: 8
                :caption: Retrieving the policy management certificates.

        """

        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        validation_args = {
            "validate_issuer": validate_issuer,
            "validate_token": validate_token,
            "validate_expiration": validate_expiration,
            "validate_not_before_time": validate_not_before_time,
            "issuer": issuer,
            "validation_slack": validation_slack,
            "validate_signature": validate_signature,
            "validation_callback": validation_callback,
        }
        # Merge validation_args with kwargs to include any additional parameters
        validation_args.update(kwargs)
        options = merge_validation_args(self._config._kwargs, validation_args)

        cert_response = self._client.policy_certificates.get(**kwargs)
        token = AttestationToken(token=cert_response.token, body_type=GeneratedPolicyCertificatesResult)

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)
        certificates = []

        cert_list = token._get_body()

        for key in cert_list.policy_certificates["keys"]:
            key_certs = [pem_from_base64(cert, "CERTIFICATE") for cert in key.x5_c]
            certificates.append(key_certs)
        return certificates, token

    @distributed_trace
    def add_policy_management_certificate(
        self,
        certificate_to_add: str,
        *,
        signing_certificate: Optional[str] = None,
        validate_signature: bool = True,
        validate_expiration: bool = True,
        validation_callback: Optional[Callable[[AttestationToken, AttestationSigner], None]] = None,
        validation_slack: float = 0.5,
        validate_not_before_time: bool = True,
        validate_issuer: bool = True,
        signing_key: Optional[str] = None,
        issuer: Optional[str] = None,
        validate_token: bool = True,
        **kwargs: Any
    ) -> Tuple[AttestationPolicyCertificateResult, AttestationToken]:
        """Adds a new policy management certificate to the set of policy management certificates for the instance.

        :param str certificate_to_add: Required. PEM encoded X.509 certificate to add to
            the list of attestation policy management certificates.
        :keyword str signing_key: PEM encoded signing Key representing the key
            associated with one of the *existing* attestation signing certificates.
        :keyword str signing_certificate: PEM encoded signing certificate which is one of
            the *existing* attestation signing certificates.
        :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
            if the token is invalid, the `validation_callback` function should throw
            an exception.
        :paramtype validation_callback: ~typing.Callable[[~azure.security.attestation.AttestationToken, ~azure.security.attestation.AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword float validation_slack: Slack time for validation - tolerance applied
            to help account for clock drift between the issuer and the current machine.
        :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.

        :return: AttestationPolicyCertificateResult object describing the status
            of the add request and the token sent from the service which
            contained the response.

        :rtype: Tuple[~azure.security.attestation.AttestationPolicyCertificateResult, ~azure.security.attestation.AttestationToken]

        The :class:`AttestationPolicyCertificatesResult` response to the
        :meth:`add_policy_management_certificate` API contains two attributes
        of interest.

        The first is `certificate_resolution`, which indicates
        whether the certificate in question is present in the set of policy
        management certificates after the operation has completed, or if it is
        absent.

        The second is the `thumbprint` of the certificate added. The `thumbprint`
        for the certificate is the SHA1 hash of the DER encoding of the
        certificate.

        .. admonition:: Example: Generating and adding a new policy management
            certificates for an isolated attestation instance.

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [BEGIN add_policy_management_certificate]
                :end-before: [END add_policy_management_certificate]
                :language: python
                :dedent: 12
                :caption: Adding a policy management certificate.

        """

        if not signing_key:
            signing_key = self._signing_key  # type: ignore
        if not signing_certificate:
            signing_certificate = self._signing_certificate  # type: ignore

        if not signing_key or not signing_certificate:
            raise ValueError("A signing certificate and key must be provided to add_policy_management_certificate.")

        # Verify that the provided certificate is a valid PEM encoded X.509 certificate
        parsed_certificate = load_pem_x509_certificate(certificate_to_add.encode("ascii"))

        jwk = JSONWebKey(
            kty="RSA",
            x5_c=[base64.b64encode(parsed_certificate.public_bytes(serialization.Encoding.DER)).decode("ascii")],
        )
        add_body = AttestationCertificateManagementBody(policy_certificate=jwk)
        cert_add_token = AttestationToken(
            body=add_body,
            signing_key=signing_key,
            signing_certificate=signing_certificate,
            body_type=AttestationCertificateManagementBody,
        )

        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        validation_args = {
            "validate_signature": validate_signature,
            "validate_expiration": validate_expiration,
            "validation_callback": validation_callback,
            "validation_slack": validation_slack,
            "validate_not_before_time": validate_not_before_time,
            "validate_issuer": validate_issuer,
            "issuer": issuer,
            "validate_token": validate_token,
        }
        # Merge validation_args with kwargs to include any additional parameters
        validation_args.update(kwargs)
        options = merge_validation_args(self._config._kwargs, validation_args)

        cert_response = self._client.policy_certificates.add(cert_add_token.to_jwt_string(), **kwargs)  # type: ignore
        token = AttestationToken(
            token=cert_response.token,
            body_type=GeneratedPolicyCertificatesModificationResult,
        )

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)
        return (
            AttestationPolicyCertificateResult._from_generated(token._get_body()),  # type: ignore
            token,
        )

    @distributed_trace
    def remove_policy_management_certificate(
        self,
        certificate_to_remove: str,
        *,
        signing_key: Optional[str] = None,
        validate_token: bool = True,
        issuer: Optional[str] = None,
        signing_certificate: Optional[str] = None,
        validate_issuer: bool = True,
        validation_callback: Optional[Callable[[AttestationToken, AttestationSigner], None]] = None,
        validation_slack: float = 0.5,
        validate_signature: bool = True,
        validate_not_before_time: bool = True,
        validate_expiration: bool = True,
        **kwargs: Any
    ) -> Tuple[AttestationPolicyCertificateResult, AttestationToken]:
        """Removes a policy management certificate from the set of policy management certificates for the instance.

        :param str certificate_to_remove: Required. PEM encoded X.509 certificate to remove from
            the list of attestation policy management certificates.
        :keyword str signing_key: PEM encoded signing Key representing the key
            associated with one of the *existing* attestation signing certificates.
        :keyword str signing_certificate: PEM encoded signing certificate which is one of
            the *existing* attestation signing certificates.
        :keyword bool validate_token: If True, validate the token, otherwise return the token unvalidated.
        :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
            if the token is invalid, the `validation_callback` function should throw
            an exception.
        :paramtype validation_callback: ~typing.Callable[[~azure.security.attestation.AttestationToken, ~azure.security.attestation.AttestationSigner], None]
        :keyword bool validate_signature: If True, validate the signature of the token being validated.
        :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
        :keyword str issuer: Expected issuer, used if `validate_issuer` is true.
        :keyword float validation_slack: Slack time for validation - tolerance applied
            to help account for clock drift between the issuer and the current machine.
        :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
        :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.
        :return: Result describing the outcome of the certificate removal.
        :rtype: Tuple[~azure.security.attestation.AttestationPolicyCertificateResult, ~azure.security.attestation.AttestationToken]

        The :class:`AttestationPolicyCertificateResult` response to the
        :meth:`remove_policy_management_certificate` API contains two attributes
        of interest.

        The first is `certificate_resolution`, which indicates
        whether the certificate in question is present in the set of policy
        management certificates after the operation has completed, or if it is
        absent.

        The second is the `thumbprint` of the certificate added. The `thumbprint`
        for the certificate is the SHA1 hash of the DER encoding of the
        certificate.

        .. admonition:: Example: Removing an added policy management
            certificate for an isolated attestation instance.

            .. literalinclude:: ../samples/sample_get_set_policy.py
                :start-after: [BEGIN remove_policy_management_certificate]
                :end-before: [END remove_policy_management_certificate]
                :language: python
                :dedent: 8
                :caption: Removing a policy management certificate.

        """

        if not signing_key:
            signing_key = self._signing_key  # type: ignore
        if not signing_certificate:
            signing_certificate = self._signing_certificate  # type: ignore

        if not signing_key or not signing_certificate:
            raise ValueError("A signing certificate and key must be provided to remove_policy_management_certificate.")

        # Verify that the provided certificate is a valid PEM encoded X.509 certificate
        parsed_certificate = load_pem_x509_certificate(certificate_to_remove.encode("ascii"))

        jwk = JSONWebKey(
            kty="RSA",
            x5_c=[base64.b64encode(parsed_certificate.public_bytes(serialization.Encoding.DER)).decode("ascii")],
        )
        add_body = AttestationCertificateManagementBody(policy_certificate=jwk)
        cert_add_token = AttestationToken(
            body=add_body,
            signing_key=signing_key,
            signing_certificate=signing_certificate,
            body_type=AttestationCertificateManagementBody,
        )
        # Merge our existing config options with the options for this API call.
        # Note that this must be done before calling into the implementation
        # layer because the implementation layer doesn't like keyword args that
        # it doesn't expect :(.
        validation_args = {
            "validate_token": validate_token,
            "issuer": issuer,
            "validate_issuer": validate_issuer,
            "validation_callback": validation_callback,
            "validation_slack": validation_slack,
            "validate_signature": validate_signature,
            "validate_not_before_time": validate_not_before_time,
            "validate_expiration": validate_expiration,
        }
        # Merge validation_args with kwargs to include any additional parameters
        validation_args.update(kwargs)
        options = merge_validation_args(self._config._kwargs, validation_args)

        cert_response = self._client.policy_certificates.remove(cert_add_token.to_jwt_string(), **kwargs)  # type: ignore
        token = AttestationToken(
            token=cert_response.token,
            body_type=GeneratedPolicyCertificatesModificationResult,
        )

        if options.get("validate_token", True):
            token._validate_token(self._get_signers(**kwargs), **options)
        return (
            AttestationPolicyCertificateResult._from_generated(token._get_body()),  # type: ignore
            token,
        )

    def _get_signers(self, **kwargs: Any) -> List[AttestationSigner]:
        """Returns the set of signing certificates used to sign attestation tokens.

        :return: A list of attestation signers used to validate tokens.
        :rtype: List[AttestationSigner]
        """

        with self._statelock:
            if not self._signing_certificates:
                signing_certificates = self._client.signing_certificates.get(**kwargs)
                self._signing_certificates = []
                for key in signing_certificates["keys"]:
                    # Convert the returned certificate chain into an array of X.509 Certificates.
                    self._signing_certificates.append(AttestationSigner._from_generated(key))  # type: ignore
            signers: list[AttestationSigner] = self._signing_certificates
        return signers

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> Self:
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details) -> None:
        self._client.__exit__(*exc_details)


class AttestationClientConfiguration(Configuration):
    """Configuration for AttestationClient.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :keyword bool validate_token: if True, validate the token, otherwise return the token unvalidated.
    :keyword validation_callback: Function callback to allow clients to perform custom validation of the token.
        if the token is invalid, the `validation_callback` function should throw
        an exception.
    :paramtype validation_callback: Callable[[AttestationToken, AttestationSigner], None]
    :keyword bool validate_signature: if True, validate the signature of the token being validated.
    :keyword bool validate_expiration: If True, validate the expiration time of the token being validated.
    :keyword str issuer: Expected issuer, used if validate_issuer is true.
    :keyword float validation_slack: Slack time for validation - tolerance applied
        to help account for clock drift between the issuer and the current machine.
    :keyword bool validate_issuer: If True, validate that the issuer of the token matches the expected issuer.
    :keyword bool validate_not_before_time: If true, validate the "Not Before" time in the token.
    """

    def __init__(self, **kwargs: Any) -> None:
        super(AttestationClientConfiguration, self).__init__(**kwargs)  # pylint: disable=super-with-arguments

        self._kwargs = kwargs.copy()


__all__: List[str] = [
    "AttestationClient",
    "AttestationAdministrationClient",
    "AttestationSigner",
    "AttestationToken",
    "AttestationTokenValidationException",
    "AttestationPolicyToken",
]  # Add all objects you want publicly available to users at this package level


def patch_sdk():
    """Do not remove from this file.

    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
import json
from typing import Any, Callable, Optional, TypeVar, Union

from azure.core import AsyncPipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict

from ... import models as _models
from ..._utils.model_base import SdkJSONEncoder, _deserialize
from ..._utils.serialization import Deserializer, Serializer
from ...operations._operations import (
    build_string_get_known_value_request,
    build_string_put_known_value_request,
    build_string_put_unknown_value_request,
)
from .._configuration import FixedClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class StringOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~typetest.enum.fixed.aio.FixedClient`'s
        :attr:`string` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: FixedClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace_async
    async def get_known_value(self, **kwargs: Any) -> Union[str, _models.DaysOfWeekEnum]:
        """getKnownValue.

        :return: DaysOfWeekEnum
        :rtype: str or ~typetest.enum.fixed.models.DaysOfWeekEnum
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[Union[str, _models.DaysOfWeekEnum]] = kwargs.pop("cls", None)

        _request = build_string_get_known_value_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _decompress = kwargs.pop("decompress", True)
        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    await response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["content-type"] = self._deserialize("str", response.headers.get("content-type"))

        if _stream:
            deserialized = response.iter_bytes() if _decompress else response.iter_raw()
        else:
            deserialized = _deserialize(Union[str, _models.DaysOfWeekEnum], response.json())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace_async
    async def put_known_value(self, body: Union[str, _models.DaysOfWeekEnum], **kwargs: Any) -> None:
        """putKnownValue.

        :param body: _. Known values are: "Monday", "Tuesday", "Wednesday", "Thursday", "Friday",
         "Saturday", and "Sunday". Required.
        :type body: str or ~typetest.enum.fixed.models.DaysOfWeekEnum
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("content-type", "application/json"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_string_put_known_value_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace_async
    async def put_unknown_value(self, body: Union[str, _models.DaysOfWeekEnum], **kwargs: Any) -> None:
        """putUnknownValue.

        :param body: _. Known values are: "Monday", "Tuesday", "Wednesday", "Thursday", "Friday",
         "Saturday", and "Sunday". Required.
        :type body: str or ~typetest.enum.fixed.models.DaysOfWeekEnum
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: str = kwargs.pop("content_type", _headers.pop("content-type", "application/json"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_string_put_unknown_value_request(
            content_type=content_type,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.textanalytics import TextAnalysisClient
from azure.ai.textanalytics.models import (
    TextLanguageDetectionInput,
    LanguageDetectionTextInput,
    LanguageInput,
    AnalyzeTextLanguageDetectionResult,
    LanguageDetectionDocumentResult,
)

TextAnalysisPreparer = functools.partial(
    EnvironmentVariableLoader,
    "text_analysis",
    text_analysis_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    text_analysis_key="fake_key",
)


class TestTextAnalysis(AzureRecordedTestCase):
    def create_client(self, endpoint: str, key: str) -> TextAnalysisClient:
        return TextAnalysisClient(endpoint, AzureKeyCredential(key))


class TestTextAnalysisCase(TestTextAnalysis):
    @TextAnalysisPreparer()
    @recorded_by_proxy
    def test_language_detection(self, text_analysis_endpoint, text_analysis_key):
        client = self.create_client(text_analysis_endpoint, text_analysis_key)

        text_a = "Sentences in different languages."

        body = TextLanguageDetectionInput(
            text_input=LanguageDetectionTextInput(language_inputs=[LanguageInput(id="A", text=text_a)])
        )

        result = client.analyze_text(body=body)

        assert result is not None
        assert isinstance(result, AnalyzeTextLanguageDetectionResult)

        assert result.results is not None
        assert result.results.documents is not None

        for doc in result.results.documents:
            assert isinstance(doc, LanguageDetectionDocumentResult)
            assert doc.id is not None
            assert doc.detected_language is not None
            assert doc.detected_language.name is not None
            assert doc.detected_language.iso6391_name is not None
            assert doc.detected_language.confidence_score is not None



# pylint: disable=too-many-lines,line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
Async tests for ResponsesInstrumentor with File Search tool.
"""

import os
from io import BytesIO
import pytest
from gen_ai_trace_verifier import GenAiTraceVerifier  # pylint: disable=import-error
from devtools_testutils.aio import recorded_by_proxy_async
from devtools_testutils import RecordedTransport
from test_base import servicePreparer
from test_ai_instrumentor_base import (  # pylint: disable=import-error
    TestAiAgentsInstrumentorBase,
    CONTENT_TRACING_ENV_VARIABLE,
)
from azure.ai.projects.telemetry import AIProjectInstrumentor, _utils
from azure.ai.projects.telemetry._utils import (
    OPERATION_NAME_INVOKE_AGENT,
    SPAN_NAME_INVOKE_AGENT,
    _set_use_message_events,
    RESPONSES_PROVIDER,
)
from azure.ai.projects.models import PromptAgentDefinition, FileSearchTool
from azure.core.settings import settings

settings.tracing_implementation = "OpenTelemetry"
_utils._span_impl_type = settings.tracing_implementation()  # pylint: disable=not-callable


class TestResponsesInstrumentorFileSearchAsync(TestAiAgentsInstrumentorBase):
    """Async tests for ResponsesInstrumentor with File Search tool."""

    # pylint: disable=too-many-nested-blocks

    @pytest.mark.usefixtures("instrument_with_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_file_search_non_streaming_with_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
        """Test asynchronous File Search agent with non-streaming and content recording enabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "True",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create product information document
            product_info = """Contoso Galaxy Innovations SmartView Glasses

Product Category: Smart Eyewear

Key Features:
- Augmented Reality interface
- Voice-controlled AI agent
- HD video recording with 3D audio
- UV protection and blue light filtering
- Wireless charging with extended battery life

Warranty: Two-year limited warranty on electronic components
Return Policy: 30-day return policy with no questions asked
"""

            # Create vector store and upload document
            vector_store = await openai_client.vector_stores.create(name="ProductInfoStore")

            product_file = BytesIO(product_info.encode("utf-8"))
            product_file.name = "product_info.txt"

            file = await openai_client.vector_stores.files.upload_and_poll(
                vector_store_id=vector_store.id,
                file=product_file,
            )

            assert file.status == "completed", f"File upload failed with status: {file.status}"

            # Create agent with File Search tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can search through uploaded documents to answer questions.",
                    tools=[FileSearchTool(vector_store_ids=[vector_store.id])],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers file search
                response = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Tell me about Contoso products",
                    stream=False,
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                assert response.output_text is not None

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass  # Just iterate to consume items

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response.id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content IS present
                from collections.abc import Mapping
                import json

                found_file_search_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        # Validate input text content IS present
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part and isinstance(
                                            part["content"], str
                                        ), "Text content should be present when content recording is enabled"
                                        assert "Contoso" in part["content"], "Should contain the user query"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_call = True
                                            assert "id" in tool_content, "file_search_call should have id"
                                            # With content recording, queries should be present
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" in file_search
                                                ), "queries should be present in file_search when content recording is enabled"
                                                queries = file_search["queries"]
                                                assert (
                                                    isinstance(queries, list) and len(queries) > 0
                                                ), "queries should be a non-empty list"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" in part
                                        ), "text content should be present when content recording is enabled"
                                        assert (
                                            isinstance(part["content"], str) and len(part["content"]) > 0
                                        ), "text content should be non-empty"

                assert found_file_search_call, "Should have found file_search_call in output"
                assert found_text_response, "Should have found text response in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_file_search_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part, "text content should be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "file_search_call should have id in conversation items"
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" in file_search
                                                ), "queries should be present when content recording is enabled"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_file_search_in_items, "Should have found file_search_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.vector_stores.delete(vector_store.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)

    @pytest.mark.usefixtures("instrument_without_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_file_search_non_streaming_without_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
        """Test asynchronous File Search agent with non-streaming and content recording disabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "False",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert not AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create product information document
            product_info = """Contoso Galaxy Innovations SmartView Glasses

Product Category: Smart Eyewear

Key Features:
- Augmented Reality interface
- Voice-controlled AI agent
- HD video recording with 3D audio
- UV protection and blue light filtering
- Wireless charging with extended battery life

Warranty: Two-year limited warranty on electronic components
Return Policy: 30-day return policy with no questions asked
"""

            # Create vector store and upload document
            vector_store = await openai_client.vector_stores.create(name="ProductInfoStore")

            product_file = BytesIO(product_info.encode("utf-8"))
            product_file.name = "product_info.txt"

            file = await openai_client.vector_stores.files.upload_and_poll(
                vector_store_id=vector_store.id,
                file=product_file,
            )

            assert file.status == "completed", f"File upload failed with status: {file.status}"

            # Create agent with File Search tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can search through uploaded documents to answer questions.",
                    tools=[FileSearchTool(vector_store_ids=[vector_store.id])],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers file search
                response = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Tell me about Contoso products",
                    stream=False,
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                assert response.output_text is not None

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass  # Just iterate to consume items

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response.id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content is NOT present
                from collections.abc import Mapping
                import json

                found_file_search_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        # Validate input text content is NOT present
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "Text content should NOT be present when content recording is disabled"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_call = True
                                            assert "id" in tool_content, "file_search_call should have id"
                                            # Without content recording, queries should NOT be present
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" not in file_search
                                                ), "queries should NOT be present when content recording is disabled"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present when content recording is disabled"

                assert found_file_search_call, "Should have found file_search_call in output"
                assert found_text_response, "Should have found text response type in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_file_search_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "file_search_call should have id in conversation items"
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" not in file_search
                                                ), "queries should NOT be present when content recording is disabled"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_file_search_in_items, "Should have found file_search_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.vector_stores.delete(vector_store.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)

    # ========================================
    # Async File Search Agent Tests - Streaming
    # ========================================

    @pytest.mark.usefixtures("instrument_with_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_file_search_streaming_with_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
        """Test asynchronous File Search agent with streaming and content recording enabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "True",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create product information document
            product_info = """Contoso Galaxy Innovations SmartView Glasses

Product Category: Smart Eyewear

Key Features:
- Augmented Reality interface
- Voice-controlled AI agent
- HD video recording with 3D audio
- UV protection and blue light filtering
- Wireless charging with extended battery life

Warranty: Two-year limited warranty on electronic components
Return Policy: 30-day return policy with no questions asked
"""

            # Create vector store and upload document
            vector_store = await openai_client.vector_stores.create(name="ProductInfoStore")

            product_file = BytesIO(product_info.encode("utf-8"))
            product_file.name = "product_info.txt"

            file = await openai_client.vector_stores.files.upload_and_poll(
                vector_store_id=vector_store.id,
                file=product_file,
            )

            assert file.status == "completed", f"File upload failed with status: {file.status}"

            # Create agent with File Search tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can search through uploaded documents to answer questions.",
                    tools=[FileSearchTool(vector_store_ids=[vector_store.id])],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers file search with streaming
                stream = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Tell me about Contoso products",
                    stream=True,
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                # Consume the stream
                async for event in stream:
                    pass

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                assert span.attributes is not None
                response_id = span.attributes.get("gen_ai.response.id")
                assert response_id is not None

                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response_id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content IS present
                from collections.abc import Mapping
                import json

                found_file_search_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part and isinstance(
                                            part["content"], str
                                        ), "Text content should be present when content recording is enabled"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_call = True
                                            assert "id" in tool_content, "file_search_call should have id"
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" in file_search
                                                ), "queries should be present when content recording is enabled"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" in part
                                        ), "text content should be present when content recording is enabled"

                assert found_file_search_call, "Should have found file_search_call in output"
                assert found_text_response, "Should have found text response in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_file_search_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part, "text content should be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "file_search_call should have id in conversation items"
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" in file_search
                                                ), "queries should be present when content recording is enabled"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_file_search_in_items, "Should have found file_search_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.vector_stores.delete(vector_store.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)

    @pytest.mark.usefixtures("instrument_without_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_file_search_streaming_without_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements
        """Test asynchronous File Search agent with streaming and content recording disabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "False",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert not AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create product information document
            product_info = """Contoso Galaxy Innovations SmartView Glasses

Product Category: Smart Eyewear

Key Features:
- Augmented Reality interface
- Voice-controlled AI agent
- HD video recording with 3D audio
- UV protection and blue light filtering
- Wireless charging with extended battery life

Warranty: Two-year limited warranty on electronic components
Return Policy: 30-day return policy with no questions asked
"""

            # Create vector store and upload document
            vector_store = await openai_client.vector_stores.create(name="ProductInfoStore")

            product_file = BytesIO(product_info.encode("utf-8"))
            product_file.name = "product_info.txt"

            file = await openai_client.vector_stores.files.upload_and_poll(
                vector_store_id=vector_store.id,
                file=product_file,
            )

            assert file.status == "completed", f"File upload failed with status: {file.status}"

            # Create agent with File Search tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can search through uploaded documents to answer questions.",
                    tools=[FileSearchTool(vector_store_ids=[vector_store.id])],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers file search with streaming
                stream = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Tell me about Contoso products",
                    stream=True,
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                # Consume the stream
                async for event in stream:
                    pass

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                assert span.attributes is not None
                response_id = span.attributes.get("gen_ai.response.id")
                assert response_id is not None

                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response_id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content is NOT present
                from collections.abc import Mapping
                import json

                found_file_search_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "Text content should NOT be present when content recording is disabled"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_call = True
                                            assert "id" in tool_content, "file_search_call should have id"
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" not in file_search
                                                ), "queries should NOT be present when content recording is disabled"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present when content recording is disabled"

                assert found_file_search_call, "Should have found file_search_call in output"
                assert found_text_response, "Should have found text response type in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_file_search_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "file_search_call":
                                            found_file_search_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "file_search_call should have id in conversation items"
                                            file_search = tool_content.get("file_search")
                                            if file_search:
                                                assert (
                                                    "queries" not in file_search
                                                ), "queries should NOT be present when content recording is disabled"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_file_search_in_items, "Should have found file_search_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.vector_stores.delete(vector_store.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)



# coding=utf-8
from collections.abc import MutableMapping
from io import IOBase
import json
from typing import Any, Callable, IO, Optional, TypeVar, Union, overload

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from corehttp.paging import AsyncItemPaged, AsyncList
from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient
from corehttp.runtime.pipeline import PipelineResponse
from corehttp.utils import case_insensitive_dict

from ... import models as _models2
from ..... import models as _models4
from ....._utils.model_base import SdkJSONEncoder, _deserialize
from ....._utils.serialization import Deserializer, Serializer
from .....aio._configuration import PageableClientConfiguration
from ...operations._operations import build_server_driven_pagination_alternate_initial_verb_post_request

JSON = MutableMapping[str, Any]
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class ServerDrivenPaginationAlternateInitialVerbOperations:  # pylint: disable=name-too-long
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~payload.pageable.aio.PageableClient`'s
        :attr:`alternate_initial_verb` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: AsyncPipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: PageableClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    def post(
        self, body: _models2.Filter, *, content_type: str = "application/json", **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """post.

        :param body: Required.
        :type body: ~payload.pageable.serverdrivenpagination.alternateinitialverb.models.Filter
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An iterator like instance of Pet
        :rtype: ~corehttp.paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def post(
        self, body: JSON, *, content_type: str = "application/json", **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """post.

        :param body: Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An iterator like instance of Pet
        :rtype: ~corehttp.paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    @overload
    def post(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> AsyncItemPaged["_models4.Pet"]:
        """post.

        :param body: Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An iterator like instance of Pet
        :rtype: ~corehttp.paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~corehttp.exceptions.HttpResponseError:
        """

    def post(self, body: Union[_models2.Filter, JSON, IO[bytes]], **kwargs: Any) -> AsyncItemPaged["_models4.Pet"]:
        """post.

        :param body: Is one of the following types: Filter, JSON, IO[bytes] Required.
        :type body: ~payload.pageable.serverdrivenpagination.alternateinitialverb.models.Filter or JSON
         or IO[bytes]
        :return: An iterator like instance of Pet
        :rtype: ~corehttp.paging.AsyncItemPaged[~payload.pageable.models.Pet]
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[list[_models4.Pet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})
        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_server_driven_pagination_alternate_initial_verb_post_request(
                    content_type=content_type,
                    content=_content,
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            else:
                _request = HttpRequest("GET", next_link, headers=_headers)
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            return _request

        async def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(
                list[_models4.Pet],
                deserialized.get("pets", []),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("next") or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client.pipeline.run(_request, stream=_stream, **kwargs)
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum. Indicates the action type. "Internal" refers to actions that are for internal only APIs."""

    INTERNAL = "Internal"


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity that created the resource."""

    USER = "User"
    APPLICATION = "Application"
    MANAGED_IDENTITY = "ManagedIdentity"
    KEY = "Key"


class ExperimentActionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum union of Chaos experiment action types."""

    DELAY = "delay"
    DISCRETE = "discrete"
    CONTINUOUS = "continuous"


class FilterType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum that discriminates between filter types. Currently only ``Simple`` type is supported."""

    SIMPLE = "Simple"
    """Simple filter type."""


class ManagedServiceIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of managed service identity (where both SystemAssigned and UserAssigned types are
    allowed).
    """

    NONE = "None"
    SYSTEM_ASSIGNED = "SystemAssigned"
    USER_ASSIGNED = "UserAssigned"
    SYSTEM_ASSIGNED_USER_ASSIGNED = "SystemAssigned,UserAssigned"


class Origin(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit
    logs UX. Default value is "user,system".
    """

    USER = "user"
    SYSTEM = "system"
    USER_SYSTEM = "user,system"


class ProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Current provisioning state for a given Azure Chaos resource."""

    SUCCEEDED = "Succeeded"
    """Resource has been created."""
    FAILED = "Failed"
    """Resource creation failed."""
    CANCELED = "Canceled"
    """Resource creation was canceled."""
    CREATING = "Creating"
    """Initial creation in progress."""
    UPDATING = "Updating"
    """Update in progress."""
    DELETING = "Deleting"
    """Deletion in progress."""


class SelectorType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum of the selector type."""

    LIST = "List"
    """List selector type."""
    QUERY = "Query"
    """Query selector type."""


class TargetReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum of the Target reference type."""

    CHAOS_TARGET = "ChaosTarget"
    """Chaos target reference type."""



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from devtools_testutils.aio import recorded_by_proxy_async
from testpreparer import DocumentationPreparer
from testpreparer_async import DocumentationClientTestBaseAsync


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDocumentationListsOperationsAsync(DocumentationClientTestBaseAsync):
    @DocumentationPreparer()
    @recorded_by_proxy_async
    async def test_lists_bullet_points_op(self, documentation_endpoint):
        client = self.create_async_client(endpoint=documentation_endpoint)
        response = await client.lists.bullet_points_op()

        # please add some check logic here by yourself
        # ...

    @DocumentationPreparer()
    @recorded_by_proxy_async
    async def test_lists_bullet_points_model(self, documentation_endpoint):
        client = self.create_async_client(endpoint=documentation_endpoint)
        response = await client.lists.bullet_points_model(
            body={"input": {"prop": "str"}},
            input={"prop": "str"},
        )

        # please add some check logic here by yourself
        # ...

    @DocumentationPreparer()
    @recorded_by_proxy_async
    async def test_lists_numbered(self, documentation_endpoint):
        client = self.create_async_client(endpoint=documentation_endpoint)
        response = await client.lists.numbered()

        # please add some check logic here by yourself
        # ...



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from tornado import httpclient

from devtools_testutils.perfstress_tests import PerfStressTest


class TornadoGetTest(PerfStressTest):
    async def global_setup(self):
        httpclient.AsyncHTTPClient.configure("tornado.curl_httpclient.CurlAsyncHTTPClient")
        type(self).client = httpclient.AsyncHTTPClient()

    async def run_async(self):
        await type(self).client.fetch(self.args.url)

    @staticmethod
    def add_arguments(parser):
        parser.add_argument("-u", "--url", required=True)
